--- win32reg.erl.old	2006-08-03 17:38:28.666473600 -0400
+++ win32reg.erl	2006-08-03 18:18:42.287089600 -0400
@@ -21,6 +21,8 @@
 	 current_key/1, change_key/2, change_key_create/2,
 	 sub_keys/1, delete_key/1,
 	 value/2, values/1, set_value/3, delete_value/2,
+	 value_typed/2, values_typed/1,
+	 set_value_typed/4,
 	 expand/1,
 	 format_error/1]).
 
@@ -45,10 +47,12 @@
 -define(cmd_delete_value, 8).
 
 %% Data types.
--define(reg_sc, 1).
--define(reg_expand_sc, 2).
+-define(reg_none, 0).
+-define(reg_sz, 1).
+-define(reg_expand_sz, 2).
 -define(reg_binary, 3).
 -define(reg_dword, 4).
+-define(reg_multi_sz, 7).
 
 %%% Exported functions.
 
@@ -117,11 +121,55 @@
     Reg ! {self(), {command, Cmd}},
     get_result(Reg).
 
+set_value_typed({win32reg, Reg}, Name0, Value, Type0) when port(Reg) ->
+    Type = type_to_type(Type0),
+    if
+        Type == ?reg_dword, integer(Value) ->
+	    set_value_typed1({win32reg, Reg}, Name0, Value, Type);
+        Type == ?reg_sz, list(Value) ->
+	    set_value_typed1({win32reg, Reg}, Name0, Value, Type);
+        Type == ?reg_expand_sz, list(Value) ->
+	    set_value_typed1({win32reg, Reg}, Name0, Value, Type);
+        Type == ?reg_multi_sz, binary(Value) ->
+	    set_value_typed1({win32reg, Reg}, Name0, Value, Type);
+        Type == ?reg_binary, binary(Value) ->
+	    set_value_typed1({win32reg, Reg}, Name0, Value, Type);
+	true ->
+	    {error, unknowntype}
+    end.
+	
+set_value_typed1({win32reg, Reg}, Name0, Value, Type0) ->
+    Type = i32(Type0),
+    Name =
+	case Name0 of
+	    default -> [];
+	    _ -> Name0
+	end,
+    {_, V} = term_to_value(Value),
+    Cmd = [?cmd_set_value, Type, Name, 0, V],
+    Reg ! {self(), {command, Cmd}},
+    get_result(Reg).
+
+value_typed({win32reg, Reg}, Name) when port(Reg) ->
+    Cmd = [?cmd_get_value, Name, 0],
+    Reg ! {self(), {command, Cmd}},
+    case get_result(Reg) of
+	{value, {Name, Value, Type}} ->
+	    {ok, Value, Type};
+	{error, Reason} ->
+	    {error, Reason}
+    end.
+    
+values_typed({win32reg, Reg}) when port(Reg) ->
+    Cmd = [?cmd_get_all_values],
+    Reg ! {self(), {command, Cmd}},
+    collect_values_typed(Reg, []).
+
 value({win32reg, Reg}, Name) when port(Reg) ->
     Cmd = [?cmd_get_value, Name, 0],
     Reg ! {self(), {command, Cmd}},
     case get_result(Reg) of
-	{value, {Name, Value}} ->
+	{value, {Name, Value, _}} ->
 	    {ok, Value};
 	{error, Reason} ->
 	    {error, Reason}
@@ -169,12 +217,22 @@
 					 
 %%% Implementation.
 
-collect_values(P, Result) ->
+collect_values_typed(P, Result) ->
     case get_result(P) of
 	ok ->
 	    {ok, lists:reverse(Result)};
 	{value, ValueData} ->
-	    collect_values(P, [ValueData|Result]);
+	    collect_values_typed(P, [ValueData|Result]);
+	{error, Reason} ->
+	    {error, Reason}
+    end.
+
+collect_values(P, Result) ->
+    case get_result(P) of
+	ok ->
+	    {ok, lists:reverse(Result)};
+	{value, {Name, Value, _}} ->
+	    collect_values(P, [{Name, Value}|Result]);
 	{error, Reason} ->
 	    {error, Reason}
     end.
@@ -209,24 +267,54 @@
 	    [] -> default;
 	    _ ->  Name0
 	end,
-    {value, {Name, encode_value(Type, Value)}};
+    {value, {Name, encode_value(Type, Value), type_to_atom(Type)}};
 get_result1([$s|Rest0]) ->
     {ok, Hkey, Name} = i32_on_head(Rest0),
     {state, Hkey, Name}.
 
-encode_value(?reg_sc, Value) ->
+type_to_atom(?reg_sz) ->
+    reg_sz;
+type_to_atom(?reg_expand_sz) ->
+    reg_expand_sz;
+type_to_atom(?reg_multi_sz) ->
+    reg_multi_sz;
+type_to_atom(?reg_dword) ->
+    reg_dword;
+type_to_atom(?reg_binary) ->
+    reg_binary;
+type_to_atom(?reg_none) ->
+    reg_none;
+type_to_atom(_Type) ->
+    {unknowntype, _Type}.
+
+type_to_type(reg_sz) ->
+    ?reg_sz;
+type_to_type(reg_expand_sz) ->
+    ?reg_expand_sz;
+type_to_type(reg_multi_sz) ->
+    ?reg_multi_sz;
+type_to_type(reg_dword) ->
+    ?reg_dword;
+type_to_type(reg_binary) ->
+    ?reg_binary;
+type_to_type(reg_none) ->
+    ?reg_none;
+type_to_type(_Type) ->
+    {unknowntype, _Type}.
+
+encode_value(?reg_sz, Value) ->
     Value;
-encode_value(?reg_expand_sc, Value) ->
+encode_value(?reg_expand_sz, Value) ->
     Value;
 encode_value(?reg_dword, Value) ->
     i32(Value);
-encode_value(_, Value) ->
+encode_value(_Type, Value) ->
     list_to_binary(Value).
 
 term_to_value(Int) when integer(Int) ->
     {i32(?reg_dword), i32(Int)};
 term_to_value(String) when list(String) ->
-    {i32(?reg_sc), [String, 0]};
+    {i32(?reg_sz), [String, 0]};
 term_to_value(Bin) when binary(Bin) ->
     {i32(?reg_binary), Bin};
 term_to_value(_) ->
